package com.cim.idm.service;

import com.cim.idm.model.dto.delivery.DeliveryDelDto;
import com.cim.idm.model.dto.delivery.DeliveryDto;
import com.cim.idm.model.dto.delivery.DeliveryEditDto;
import com.cim.idm.model.dto.delivery.StoreDetailDto;
import com.cim.idm.model.po.delivery.MaterialDelivery;
import com.cim.idm.model.po.delivery.MaterialDeliveryReceive;
import com.cim.idm.model.po.MaterialReceiveRequest;
import com.cim.idm.model.dto.PageDto;
import com.cim.idm.model.vo.DeliveryReceiveDetailVo;
import com.cim.idm.model.vo.MaterialReceiveRequestVo;
import com.cim.idm.model.vo.ReceiveDetailVo;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.Map;

public interface IDeliveryService {
    /**
     * 新增到货单
     * @param dto 请求
     * @return 到货单
     */
    int createDelivery(DeliveryDto dto);

    /**
     * 获取采购订单
     * @param dto 请求
     * @return 采购订单
     */
    PageInfo<MaterialReceiveRequestVo> getReceiveList(PageDto dto);

    /**
     * 获取采购单明细及批次
     * @param documentName 单据名称
     * @param siteName 组织
     * @param nameList 名称
     * @param deliveryName 到库单号
     * @return 采购订单
     */
    ReceiveDetailVo getStoreDetail(String documentName, String siteName, List<String> nameList, String deliveryName);

    /**
     * 获取辅材明细及批次
     * @param documentName 单据名称
     * @param siteName 组织
     * @param nameList 名称
     * @param deliveryName 到库单号
     * @return 采购订单
     */
    List<Map<String, Object>> getMaterialDetail(String documentName, String siteName, List<String> nameList, String deliveryName);

    /**
     * 根据 key 获取值
     * @param storeDetailDto 单据请求
     * @return 类型
     */
    MaterialDelivery getByKey(StoreDetailDto storeDetailDto);

    /**
     * 根据 key 获取采购单
     * @param storeDetailDto 单据请求
     * @return 类型
     */
    MaterialReceiveRequest getReceiveByKey(StoreDetailDto storeDetailDto);

    /**
     * 根据到库单获取关系
     * @param storeDetailDto 到库单
     * @return 采购单名称列表
     */
    List<MaterialDeliveryReceive> getRelByDelivery(StoreDetailDto storeDetailDto);

    /**
     * 根据到库单获取采购单名称
     * @param storeDetailDto 到库单
     * @return 采购单名称列表
     */
    List<MaterialReceiveRequest> getReceiveByDelivery(StoreDetailDto storeDetailDto);


    /**
     * 根据单号和物料获取单据详情
     * @param documentName 单据
     * @param materialSpec 物料
     * @param isDelivery 是否到库
     * @return 单据详情
     */
    List<DeliveryReceiveDetailVo> getReceiveByMaterialSpec(String documentName, String materialSpec, Boolean isDelivery);

    /**
     * 根据到库单获取采购单名称
     * @param nameList 到库单
     * @return 采购单名称列表
     */
    List<DeliveryReceiveDetailVo> getReceiveDetail(List<String> nameList);

    /**
     * 获取到货单
     * @param dto 请求
     * @return 采购订单
     */
    PageInfo<MaterialDelivery> getDeliveryList(PageDto dto);

    /**
     * 删除到货单
     * @param deliveryDelDto 请求
     * @return 结果
     */
    int delDelivery(DeliveryDelDto deliveryDelDto);

    /**
     * 更新到货单
     * @param deliveryEditDto 请求
     * @return 结果
     */
    int editDelivery(DeliveryEditDto deliveryEditDto);

}
